@@ -2,6 +2,7 @@
 
 import hashlib
 import io
+import mimetypes
 from os.path import basename
 from urllib.parse import unquote, urlsplit
 
@@ -150,7 +151,7 @@ def add_inputs(inputs, matrix, pdf, page, resources, stream, font_map,
                 'Subtype': '/Widget',
                 'Rect': pydyf.Array(rectangle),
                 'FT': '/Btn',
-                'F': 2 ** (3 - 1),  # Print flag
+                'F': 1 << (3 - 1),  # Print flag
                 'P': page.reference,
                 'T': pydyf.String(input_name),
                 'V': '/Yes' if checked else '/Off',
@@ -160,6 +161,43 @@ def add_inputs(inputs, matrix, pdf, page, resources, stream, font_map,
                 'AS': '/Yes' if checked else '/Off',
                 'DA': pydyf.String(b' '.join(field_stream.stream)),
             })
+        elif element.tag == 'select':
+            # Select fields
+            font_description = get_font_description(style)
+            font = pango.pango_font_map_load_font(
+                font_map, context, font_description)
+            font = stream.add_font(font)
+            font.used_in_forms = True
+
+            field_stream.set_font_size(font.hash, font_size)
+            options = []
+            selected_values = []
+            for option in element:
+                value = pydyf.String(option.attrib.get('value', ''))
+                text = pydyf.String(option.text)
+                options.append(pydyf.Array([value, text]))
+                if 'selected' in option.attrib:
+                    selected_values.append(value)
+
+            field = pydyf.Dictionary({
+                'DA': pydyf.String(b' '.join(field_stream.stream)),
+                'F': 1 << (3 - 1),  # Print flag
+                'FT': '/Ch',
+                'Opt': pydyf.Array(options),
+                'P': page.reference,
+                'Rect': pydyf.Array(rectangle),
+                'Subtype': '/Widget',
+                'T': pydyf.String(input_name),
+                'Type': '/Annot',
+            })
+            if 'multiple' in element.attrib:
+                field['Ff'] = 1 << (22 - 1)
+                field['V'] = pydyf.Array(selected_values)
+            else:
+                field['Ff'] = 1 << (18 - 1)
+                field['V'] = (
+                    selected_values[-1] if selected_values
+                    else pydyf.String(''))
         else:
             # Text, password, textarea, files, and unknown
             font_description = get_font_description(style)
@@ -177,18 +215,18 @@ def add_inputs(inputs, matrix, pdf, page, resources, stream, font_map,
                 'Subtype': '/Widget',
                 'Rect': pydyf.Array(rectangle),
                 'FT': '/Tx',
-                'F': 2 ** (3 - 1),  # Print flag
+                'F': 1 << (3 - 1),  # Print flag
                 'P': page.reference,
                 'T': pydyf.String(input_name),
-                'V': pydyf.String(value),
+                'V': pydyf.String(value or ''),
                 'DA': pydyf.String(b' '.join(field_stream.stream)),
             })
             if element.tag == 'textarea':
-                field['Ff'] = 2 ** (13 - 1)
+                field['Ff'] = 1 << (13 - 1)
             elif input_type == 'password':
-                field['Ff'] = 2 ** (14 - 1)
+                field['Ff'] = 1 << (14 - 1)
             elif input_type == 'file':
-                field['Ff'] = 2 ** (21 - 1)
+                field['Ff'] = 1 << (21 - 1)
 
         pdf.add_object(field)
         page['Annots'].append(field.reference)
@@ -215,7 +253,7 @@ def add_annotations(links, matrix, document, pdf, page, annot_files, compress):
             # above about multiple regions won't always be correct, because
             # two links might have the same href, but different titles.
             annot_files[annot_target] = write_pdf_attachment(
-                pdf, (annot_target, None), document.url_fetcher, compress)
+                pdf, Attachment(annot_target), compress)
         annot_file = annot_files[annot_target]
         if annot_file is None:
             continue
@@ -243,59 +281,61 @@ def add_annotations(links, matrix, document, pdf, page, annot_files, compress):
         page['Annots'].append(annot.reference)
 
 
-def write_pdf_attachment(pdf, attachment, url_fetcher, compress):
+def write_pdf_attachment(pdf, attachment, compress):
     """Write an attachment to the PDF stream."""
     # Attachments from document links like <link> or <a> can only be URLs.
     # They're passed in as tuples
-    url = ''
-    if isinstance(attachment, tuple):
-        url, description = attachment
-        attachment = Attachment(
-            url=url, url_fetcher=url_fetcher, description=description)
-    elif not isinstance(attachment, Attachment):
-        attachment = Attachment(guess=attachment, url_fetcher=url_fetcher)
-
+    url = None
+    uncompressed_length = 0
+    stream = b''
     try:
-        with attachment.source as (source_type, source, url, _):
+        with attachment.source as (_, source, url, _):
+            if isinstance(source, str):
+                source = source.encode()
             if isinstance(source, bytes):
                 source = io.BytesIO(source)
-            uncompressed_length = 0
-            stream = b''
-            md5 = hashlib.md5()
             for data in iter(lambda: source.read(4096), b''):
                 uncompressed_length += len(data)
-                md5.update(data)
                 stream += data
-            file_extra = pydyf.Dictionary({
-                'Type': '/EmbeddedFile',
-                'Params': pydyf.Dictionary({
-                    'CheckSum': f'<{md5.hexdigest()}>',
-                    'Size': uncompressed_length,
-                })
-            })
-            file_stream = pydyf.Stream([stream], file_extra, compress=compress)
-            pdf.add_object(file_stream)
-
     except URLFetchingError as exception:
         LOGGER.error('Failed to load attachment: %s', exception)
         return
+    attachment.md5 = hashlib.md5(stream).hexdigest()
 
     # TODO: Use the result object from a URL fetch operation to provide more
-    # details on the possible filename.
+    # details on the possible filename and MIME type.
     if url and urlsplit(url).path:
         filename = basename(unquote(urlsplit(url).path))
     else:
         filename = 'attachment.bin'
+    mime_type = mimetypes.guess_type(filename, strict=False)[0]
+    if not mime_type:
+        mime_type = 'application/octet-stream'
 
-    attachment = pydyf.Dictionary({
+    creation = pydyf.String(attachment.created.strftime('D:%Y%m%d%H%M%SZ'))
+    mod = pydyf.String(attachment.modified.strftime('D:%Y%m%d%H%M%SZ'))
+    file_extra = pydyf.Dictionary({
+        'Type': '/EmbeddedFile',
+        'Subtype': f'/{mime_type.replace("/", "#2f")}',
+        'Params': pydyf.Dictionary({
+            'CheckSum': f'<{attachment.md5}>',
+            'Size': uncompressed_length,
+            'CreationDate': creation,
+            'ModDate': mod,
+        })
+    })
+    file_stream = pydyf.Stream([stream], file_extra, compress=compress)
+    pdf.add_object(file_stream)
+
+    pdf_attachment = pydyf.Dictionary({
         'Type': '/Filespec',
         'F': pydyf.String(),
         'UF': pydyf.String(filename),
         'EF': pydyf.Dictionary({'F': file_stream.reference}),
         'Desc': pydyf.String(attachment.description or ''),
     })
-    pdf.add_object(attachment)
-    return attachment
+    pdf.add_object(pdf_attachment)
+    return pdf_attachment
 
 
 def resolve_links(pages):
